## REST API

### Login and version management

| Route            | Description                 | Method | Input                                                | Output                                                   |
|---               |---                          |---     |---                                                   |---                                                       |
| `/login`         | login (return access token) | POST   | [common.params.Login](../src/common/params/login.go) | [common.params.Token](../src/common/params/login.go)     |
| `/token/refresh` | obtain a new access token   | GET    | -                                                    | [common.params.Token](../src/common/params/login.go)     |
| `/version`       | obtain version number       | GET    | -                                                    | [common.params.Version](../src/common/params/version.go) |

### User management

| Route                     | Description            | Method | Input                                                      | Req. user cap.   | Output                                                            |
|---                        |---                     |---     |---                                                         |---               |---                                                                |
| `/users`                  | create a user          | POST   | [common.params.UserWithPwd](../src/common/params/users.go) | `USER_CREATE`    | -                                                                 |
| `/users/{email}`          | delete a user          | DELETE | -                                                          | `USER_DESTROY`   | -                                                                 |
| `/users/{email}/caps`     | set caps for a user    | PUT    | [common.params.UserSetCaps](../src/common/params/users.go) | `USER_SET_CAPS`  | -                                                                 |
| `/users/{email}/unlock`   | unlock a user          | PUT    | -                                                          | `USER_UNLOCK`    | -                                                                 |
| `/users/{email}/resetpwd` | reset a user's pwd     | PUT    | -                                                          | `USER_RESETPWD`  | [common.params.UserSetPwd](../src/common/params/users.go)         |
| `/users/pwd`              | set current user's pwd | PUT    | [common.params.UserSetPwd](../src/common/params/users.go)  | -                | -                                                                 |
| `/users`                  | list users             | GET    | -                                                          | `USER_LIST`      | [\[\]common.params.UserWithoutPwd](../src/common/params/users.go) |
| `/users/whoami`           | list current user      | GET    | -                                                          | -                | [common.params.UserWithoutPwd](../src/common/params/users.go)     |
     
- Open question: shall we forbid the deletion of a user if they still own templates or VMs?

### VM management

| Route                  | Description                                    | Method | Input | Req. user cap.   | Op. | Req. VM access cap. | Output                                                                |
|---                     |---                                             |---     |---    |---               |---  |---                  |---                                                                    |
| `/vms`                 | returns VMs that can be listed                 | GET    | -     | `VM_LIST_ANY`    | OR  | `VM_LIST`           | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/{id}`            | returns the specified VM                       | GET    | -     | `VM_LIST_ANY`    | OR  | `VM_LIST`           | [common.vm.VMNetworkSerialized](../src/common/vm/vm.go)               |
| `/vms/start`           | returns VMs that can be started                | GET    | -     | `VM_START_ANY`   | OR  | `VM_START`          | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/attach`          | returns "attach creds" for attachable VMs      | GET    | -     | `VM_ATTACH_ANY`  | OR  | `VM_ATTACH`         | [\[\]common.vm.VMAttachCredentialsSerialized](../src/common/vm/vm.go) |
| `/vms/{id}/attach`     | returns "attach creds" for the specified VM    | GET    | -     | `VM_ATTACH_ANY`  | OR  | `VM_ATTACH`         | [common.vm.VMAttachCredentialsSerialized](../src/common/vm/vm.go)     |
| `/vms/stop`            | returns VMs that can be killed/shutdown        | GET    | -     | `VM_STOP_ANY`    | OR  | `VM_STOP`           | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/reboot`          | returns VMs that can be rebooted               | GET    | -     | `VM_REBOOT_ANY`  | OR  | `VM_REBOOT`         | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/edit`            | returns VMs that can be edited                 | GET    | -     | `VM_EDIT_ANY`    | OR  | `VM_EDIT`           | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/editaccess`      | returns VMs that can have their access changed | GET    | -     | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/del`             | returns VMs that can be deleted                | GET    | -     | `VM_DESTROY_ANY` | OR  | `VM_DESTROY`        | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/exportdir`       | returns VMs that can have a dir downloaded     | GET    | -     | `VM_READFS_ANY`  | OR  | `VM_READFS`         | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |
| `/vms/importfiles`     | returns VMs that can have files upload to      | GET    | -     | `VM_WRITEFS_ANY` | OR  | `VM_WRITEFS`        | [\[\]common.vm.VMNetworkSerialized](../src/common/vm/vm.go)           |

| Route                      | Description                         | Method | Input                                                         | Req. user cap.   | Op. | Req. VM access cap. | Output                                                           |
|---                         |---                                  |---     |---                                                            |---               |---  |---                  |---                                                               |
| `/vms`                     | create a VM                         | POST   | [commmon.params.VMCreate](../src/common/params/vms.go)        | `VM_CREATE`      | -   |  -                  | [common.vm.VMNetworkSerialized](../src/common/vm/vm.go)          |
| `/vms/{id}`                | delete the VM                       | DELETE | -                                                             | `VM_DESTROY_ANY` | OR  | `VM_DESTROY`        | -                                                                |
| `/vms/{id}`                | edit the VM                         | PUT    | [common.params.VMEdit](../src/common/params/vms.go)           | `VM_EDIT_ANY`    | OR  | `VM_EDIT`           | [common.vm.VMNetworkSerialized](../src/common/vm/vm.go)          |
| `/vms/{id}/start`          | start the VM                        | PUT    | -                                                             | `VM_START_ANY`   | OR  | `VM_START`          | -                                                                |
| `/vms/{id}/startwithcreds` | start the VM with credentials       | PUT    | [common.params.VMStartWithCreds](../src/common/params/vms.go) | `VM_START_ANY`   | OR  | `VM_START`          | -                                                                |
| `/vms/{id}/spicecreds`     | returns Spice creds for the VM      | POST   | [common.params.VMAttachCreds](../src/common/params/vms.go)    | `VM_ATTACH_ANY`  | OR  | `VM_ATTACH`         | [common.vm.VMSpiceCredentialsSerialized](../src/common/vm/vm.go) |
| `/vms/spicecreds`          | returns Spice creds for a VM        | POST   | [common.params.VMAttachCreds](../src/common/params/vms.go)    | `VM_ATTACH_ANY`  | OR  | -                   | [common.vm.VMSpiceCredentialsSerialized](../src/common/vm/vm.go) |
| `/vms/{id}/stop`           | kill the VM                         | PUT    | -                                                             | `VM_STOP_ANY`    | OR  | `VM_STOP`           | -                                                                |
| `/vms/{id}/reboot`         | reboot the VM                       | PUT    | -                                                             | `VM_REBOOT_ANY`  | OR  | `VM_REBOOT`         | -                                                                |
| `/vms/{id}/shutdown`       | gracefully shutdown the VM          | PUT    | -                                                             | `VM_STOP_ANY`    | OR  | `VM_STOP`           | -                                                                |
| `/vms/{id}/access/{email}` | set the VM's VM access for the user | PUT    | [common.params.VMAddAccess](../src/common/params/vms.go)      | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     | -                                                                |
| `/vms/{id}/access/{email}` | del the VM's VM access for the user | DELETE | -                                                             | `VM_SET_ACCESS`  | AND | `VM_SET_ACCESS`     | -                                                                |
| `/vms/{id}/exportdir`      | download a dir from the VM          | GET    | [common.params.VMExportDir](../src/common/params/vms.go)      | `VM_READFS_ANY`  | OR  | `VM_READFS`         | tar.gz archive                                                   |
| `/vms/{id}/importfiles`    | upload files into a dir in the VM   | POST   | multipart form: { "vmDir" (path), "file" (tar.gz archive) }   | `VM_WRITEFS_ANY` | OR  | `VM_WRITEFS`        |                                                                  |

### Template management

| Route                  | Description                          | Method | Input                                                        | Req. user cap.                     | Output                                                                       |
|---                     |---                                   |---     |---                                                           |---                                 |---                                                                           |
| `/templates`           | returns templates that can be listed | GET    | -                                                            | `TPL_LIST_ANY` OR `TPL_LIST`       | [common.template.TemplateSerialized](../src/common/template/template.go)     |
| `/templates/{id}`      | returns a template                   | GET    | -                                                            | `TPL_LIST_ANY` OR `TPL_LIST`       | [\[\]common.template.TemplateSerialized](../src/common/template/template.go) |
| `/templates/vm`        | create a template                    | POST   | [common.params.TplCreate](../src/common/params/templates.go) | `TPL_CREATE`                       | [common.template.TemplateSerialized](../src/common/template/template.go)     |
| `/templates/{id}`      | edit a template                      | PUT    | [common.params.TplEdit](../src/common/params/templates.go)   | `TPL_EDIT_ANY` OR `TPL_EDIT`       | [common.template.TemplateSerialized](../src/common/template/template.go)     |
| `/templates/{id}`      | delete a template                    | DELETE | -                                                            | `TPL_DESTROY_ANY` OR `TPL_DESTROY` | -                                                                            |
| `/templates/{id}/disk` | download a template's disk image     | GET    | -                                                            | `TPL_READFS_ANY` OR `TPL_READFS`   | qcow file                                                                    |

<!--
| `/templates/qcow`      | create a template | POST   | multipart form: { "name", "access", "qcow" (qcow file) } | `TPL_CREATE`  | common.template.TemplateSerialized |
-->

Remarks:

- A template can only be deleted if no VM references it!

<!--
### Group management

| `/vms/group/{gid}`       | list all VMs in a group                  | GET    |         | `VM_LIST|VM_LIST_ANY`       |
| `/vms/group/{gid}`       | delete all VMs in a group                | DELETE |         | `VM_DESTROY|VM_DESTROY_ANY` |
| `/vms/group/{gid}/start` | start all VMs in a group                 | PUT    |         | `VM_START|VM_START_ANY`     |
| `/vms/group/{gid}/stop`  | stop all VMs in a group                  | PUT    |         | `VM_STOP|VM_STOP_ANY`       |
| `/vms/group/{gid}/mask`  | mask (to non-owner) all VMs in a group   | PUT    |         | `VM_MASK|VM_MASK_ANY`       |
| `/vms/group/{gid}/unmask`| unmask (to non-owner) all VMs in a group | PUT    |         | `VM_MASK|VM_MASK_ANY`       |
| `/vms/{id}/move/{gid}`   | move a VM into a group                   | PUT    |         | `VM_MOVE|VM_MOVE_ANY`       |

| Route                 | Description             | Method | Parameters | Required capabilities         |
|---                    |---                      |---     |---         |---                            |
| `/groups`             | create a group          | POST   | name       | `GRP_CREATE`                  |
| `/groups/{id}`        | delete a group          | DELETE |            | `GRP_DESTROY|GRP_DESTROY_ANY` |
| `/groups/{id}/rename` | rename a group          | PUT    | new name   | `GRP_RENAME|GRP_RENAME_ANY`   |
| `/groups`             | list groups             | GET    |            | `GRP_LIST|GRP_LIST_ANY`       |
-->
